unit xProcReg;
  { registry functions }

interface

uses
  Windows, Classes, Registry, 
  xProcs;

const
  { important registry keys / items }
  REG_CURRENT_VERSION = 'Software\Microsoft\Windows\CurrentVersion';
  REG_CURRENT_USER    = 'RegisteredOwner';
  REG_CURRENT_COMPANY = 'RegisteredOrganization';

type
  { Data types }
  TRegUtils = class
  private
    class procedure ParsePath(const Path: String; var aPath, aValue: String);
  public
    class function  ReadString(aKey: hKey; const Path: String): String;
    class procedure WriteString(aKey: hKey; const Path,Value: String);
    class function  InfoString(const Value: String): String;
    class procedure DelValue(aKey: hKey; const Path: String);

    class function  CurrentUser: String;
    class function  CurrentCompany: String;

    class procedure WriteShellExt(const aExt,aCmd,aMenu,aExec: String);

    { The following five functions came from David W. Yutzy / Celeste Software Services
      Thanks for submitting us the methods !!
    }
    class procedure KeyList(aKey: HKEY; const Path:String; var aValue: TStringList);
    class function  ValueExist(aKey: HKEY; const Path:String):Boolean;
    class function  WriteValue(aKey: HKEY; const Path: String; Value: Variant; Typ: TDataType): Boolean;
    class function  ReadValue(aKey:HKEY; const Path:String; Typ: TDataType): Variant;
    class procedure ValueList(aKey: HKEY; const Path:String; var aValue: TStringList);
  end;

  { Persistent access of components from the registry }
  TPersistentRegistry = class(TRegistry)
  public
    function  ReadComponent(const Name: String; Owner, Parent: TComponent): TComponent;
    procedure WriteComponent(const Name: String; Component: TComponent);
  end;


implementation

{ TRegUtils }

class procedure TRegUtils.ParsePath(const Path: String; var aPath, aValue: String);
begin
  aPath:=Path;
  aValue:= '';
  while (Length(aPath)>0) and (strLastCh(aPath)<>'\') do
  begin
    aValue:=strLastCh(aPath)+aValue;
    strStripLast(aPath);
  end;
end;

class function TRegUtils.ReadString(aKey: HKEY; const Path: String): String;
var
  aRegistry : TRegistry;
  aPath     : String;
  aValue    : String;
begin
  aRegistry:=TRegistry.Create;
  try
    with aRegistry do
    begin
      RootKey:=aKey;
      ParsePath(Path, aPath, aValue);
      OpenKey(aPath,True);
      Result:=ReadString(aValue);
    end;
  finally
    aRegistry.Free;
  end;
end;

class procedure TRegUtils.WriteString(aKey: HKEY; const Path,Value: String);
var
  aRegistry : TRegistry;
  aPath     : String;
  aValue    : String;
begin
  aRegistry:=TRegistry.Create;
  try
    with aRegistry do
    begin
      RootKey:=aKey;
      ParsePath(Path, aPath, aValue);
      OpenKey(aPath,True);
      WriteString(aValue,Value);
    end;
  finally
    aRegistry.Free;
  end;
end;

class procedure TRegUtils.DelValue(aKey: hKey; const Path: String);
var
  aRegistry : TRegistry;
  aPath     : String;
  aValue    : String;
begin
  aRegistry:=TRegistry.Create;
  try
    with aRegistry do
    begin
      RootKey:=aKey;
      ParsePath(Path, aPath, aValue);
      OpenKey(aPath,True);
      DeleteValue(aValue);
    end;
  finally
    aRegistry.Free;
  end;
end;

class function TRegUtils.InfoString(const Value: String): String;
var
  aKey : hKey;
begin
  Result:='';
  if RegOpenKey(HKEY_LOCAL_MACHINE,REG_CURRENT_VERSION,aKey)=ERROR_SUCCESS then
  begin
    Result:=ReadString(aKey,Value);
    RegCloseKey(aKey);
  end;
end;

class function TRegUtils.CurrentUser: String;
begin
  Result:=InfoString(REG_CURRENT_USER);
end;

class function TRegUtils.CurrentCompany: String;
begin
  Result:=InfoString(REG_CURRENT_COMPANY);
end;

{ Add a shell extension to the registry }
class procedure TRegUtils.WriteShellExt(const aExt,aCmd,aMenu,aExec: String);
var
  s, aPath : String;
begin
  with TRegistry.Create do
  try
    RootKey := HKEY_CLASSES_ROOT;
    aPath   := aExt;
    if KeyExists(aPath) then
    begin
      OpenKey(aPath,False);
      S:=ReadString('');
      CloseKey;
      if S<>'' then
         if KeyExists(S) then
            aPath:=S;
    end;

    OpenKey(aPath+'\Shell\'+aCmd,True);
    WriteString('',aMenu);
    CloseKey;

    OpenKey(aPath+'\Shell\'+aCmd+'\Command',True);
    WriteString('',aExec + ' %1');
    CloseKey;
  finally
    Free;
  end;
end;

class procedure TRegUtils.ValueList(aKey: HKEY; const Path:String; var aValue: TStringList);
var
  aRegistry: TRegistry;
begin
  aRegistry:=TRegistry.Create;
  try
    with aRegistry do
    begin
      RootKey:=aKey;
      OpenKey(Path,True);
      GetValueNames(aValue);
    end;
  finally
    aRegistry.Free;
  end;
end;

class procedure TRegUtils.KeyList(aKey: HKEY; const Path:String; var aValue: TStringList);
var
  aRegistry: TRegistry;
begin
  aRegistry:=TRegistry.Create;
  try
    with aRegistry do
    begin
      RootKey:=aKey;
      OpenKey(Path,True);
      GetKeyNames(aValue);
    end;
  finally
    aRegistry.Free;
  end;
end;

class function TRegUtils.ValueExist(aKey: HKEY; const Path:String):Boolean;
var
  aRegistry: TRegistry;
  aPath: String;
  aValue: String;
begin
  aRegistry:=TRegistry.Create;
  try
    with aRegistry do
    begin
      RootKey:=aKey;
      ParsePath(Path, aPath, aValue);
      OpenKey(aPath,True);
      Result := ValueExists(aValue)
    end;
  finally
    aRegistry.Free;
  end;
end;

class function TRegUtils.ReadValue(aKey:HKEY; const Path:String; Typ: TDataType): Variant;
var
  aRegistry: TRegistry;
  aPath: String;
  aValue: String;
begin
  aRegistry:=TRegistry.Create;
  try
    with aRegistry do
    begin
      RootKey:=aKey;
      ParsePath(Path, aPath, aValue);
      if OpenKey(aPath,True) then
        if ValueExists(aValue) then
           case Typ of
             dtInteger:  Result := ReadInteger(aValue);
             dtBoolean:  Result := ReadBool(aValue);
             dtString:   Result := ReadString(aValue);
             dtDate:     Result := ReadDate(aValue);
             dtFloat:    Result := ReadFloat(aValue);
             dtCurrency: Result := ReadCurrency(aValue);
             dtTime:     Result := REadTime(aValue);
           end;
    end;
  finally
    aRegistry.Free;
  end;
end;

class function TRegUtils.WriteValue(aKey: HKEY; const Path: String; Value: Variant; Typ: TDataType): Boolean;
var
  aRegistry : TRegistry;
  aPath     : String;
  aValue    : String;
begin
  Result := True;
  aRegistry:=TRegistry.Create;
  try
    with aRegistry do
    begin
      RootKey:=aKey;
      ParsePath(Path, aPath, aValue);
      if OpenKey(aPath,True) then
        case Typ of
          dtInteger:  WriteInteger(aValue, Value);
          dtBoolean:  WriteBool(aValue, Value);
          dtString:   WriteString(aValue, Value);
          dtDate:     WriteDate(aValue, Value);
          dtFloat:    WriteFloat(aValue, Value);
          dtCurrency: WriteCurrency(aValue, Value);
          dtTime:     WriteTime(aValue, Value);
        end
      else
        Result := False;
    end;
  finally
    aRegistry.Free;
  end;
end;

{ TPersistentRegistry }

function TPersistentRegistry.ReadComponent(
         const Name: String; Owner, Parent: TComponent): TComponent;
var
  DataSize  : Integer;
  MemStream : TMemoryStream;
  Reader    : TReader;
begin
  DataSize:=GetDataSize(Name);
  MemStream := TMemoryStream.Create;
  try
    MemStream.SetSize(DataSize);
    ReadBinaryData(Name,MemStream.Memory^,DataSize);
    MemStream.Position := 0;

    Reader := TReader.Create(MemStream, 256);
    try
      Reader.Parent := Parent;
      Result := Reader.ReadRootComponent(nil);
      if Owner <> nil then
        try
          Owner.InsertComponent(Result);
        except
          Result.Free;
          Result := nil;
          raise;
        end;
    finally
      Reader.Free;
    end;

  finally
    MemStream.Free;
  end;
end;

procedure TPersistentRegistry.WriteComponent(const Name: String; Component: TComponent);
var
  MemStream: TMemoryStream;
begin
  MemStream := TMemoryStream.Create;
  try
    MemStream.WriteComponent(Component);
    WriteBinaryData(Name, MemStream.Memory^, MemStream.Size);
  finally
    MemStream.Free;
  end;
end;


end.
